//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import NevisMobileAuthentication

struct TypedSessionProvider: Decodable {
	// MARK: Properties

	let wrapped: SessionProvider

	// MARK: Initialization

	init(wrapped: SessionProvider) {
		self.wrapped = wrapped
	}

	// MARK: Coding Keys

	enum SessionProviderType: String, Encodable {
		case cookie = "cookieSessionProvider"
		case jwt = "jwtSessionProvider"
	}

	// MARK: Decodable

	init(from decoder: Decoder) throws {
		let container = try decoder.container(keyedBy: TypedCodingKeys.self)
		let type = try container.decode(String.self, forKey: .type)
		switch type {
		case SessionProviderType.cookie.rawValue:
			wrapped = try container.decode(CookieSessionProvider.self, forKey: .data)
		case SessionProviderType.jwt.rawValue:
			wrapped = try container.decode(JwtSessionProvider.self, forKey: .data)
		default:
			throw DecodingError.dataCorrupted(
				DecodingError.Context(
					codingPath: container.codingPath,
					debugDescription: "Failed to decode session provider!"
				)
			)
		}
	}
}
